@@ -14,8 +14,8 @@
 # limitations under the License.
 # ===============================================================================
 import os.path
-from urllib.parse import quote_plus
 
+import click
 import yaml
 from requests import Session
 from jsonschema import validate, ValidationError
@@ -24,6 +24,14 @@ import re
 IDREGEX = re.compile(r"(?P<id>\(\d+\))")
 
 
+def verbose_message(msg):
+    click.secho(msg, fg="green")
+
+
+def warning(msg):
+    click.secho(msg, fg="red")
+
+
 class BaseST:
     iotid = None
     _db_obj = None
@@ -42,7 +50,9 @@ class BaseST:
                 f"Validation failed for {self.__class__.__name__}. {err}. {self._payload}"
             )
 
-    def _generate_request(self, method, query=None, entity=None, orderby=None, expand=None, limit=None):
+    def _generate_request(
+        self, method, query=None, entity=None, orderby=None, expand=None, limit=None
+    ):
         if orderby is None:
             orderby = "$orderby=id asc"
 
@@ -62,7 +72,9 @@ class BaseST:
                 params.append(orderby)
 
             if query:
-                params.append(f"$filter={quote_plus(query)}")
+                # params.append(urlencode({"$filter": query}))
+
+                params.append(f"$filter={query}")
 
             if params:
                 url = f"{url}?{'&'.join(params)}"
@@ -103,7 +115,7 @@ class BaseST:
             if resp.status_code == 200:
                 return True
 
-    def get(self, query, entity=None, pages=None, expand=None):
+    def get(self, query, entity=None, pages=None, expand=None, verbose=False):
         orderby = None
         if pages < 0:
             pages = abs(pages)
@@ -113,20 +125,30 @@ class BaseST:
             if pages:
                 if page_count >= pages:
                     return
-                print(f"getting page={page_count + 1}/{pages} for request={request}")
+                if verbose:
+                    verbose_message(f"getting page={page_count + 1}/{pages}")
+                    verbose_message("-------------- Request -----------------")
+                    verbose_message(request["url"])
+                    verbose_message("----------------------------------------")
 
             resp = self._send_request(request)
             resp = self._parse_response(request, resp)
-            for v in resp["value"]:
-                yield v
-            try:
-                next_url = resp["@iot.nextLink"]
-            except KeyError:
+            if not resp["value"]:
+                warning("no records found")
                 return
+            else:
+                for v in resp["value"]:
+                    yield v
+                try:
+                    next_url = resp["@iot.nextLink"]
+                except KeyError:
+                    return
 
             yield from get_items({"method": "get", "url": next_url}, page_count + 1)
 
-        start_request = self._generate_request("get", query=query, entity=entity, orderby=orderby, expand=expand)
+        start_request = self._generate_request(
+            "get", query=query, entity=entity, orderby=orderby, expand=expand
+        )
         yield from get_items(start_request, 0)
 
     def put(self, dry=False):
@@ -402,10 +424,8 @@ class Client:
     def get_datastreams(self, query=None):
         yield from Datastreams(None, self._session, self._connection).get(query)
 
-    def get_locations(self, query=None, pages=None, expand=None):
-        yield from Locations(None, self._session, self._connection).get(
-            query, pages=pages, expand=expand
-        )
+    def get_locations(self, query=None, **kw):
+        yield from Locations(None, self._session, self._connection).get(query, **kw)
 
     def get_things(self, query=None, entity=None):
         yield from Things(None, self._session, self._connection).get(query, entity)
